(function() {
  function AEVMEEPROM() {}

  const CONTROLLER_NAME = "aevm";

  const BLOCK_SIZE = 32; /* used to read or write the EEPROM in blocks*/
  const EEPROM_SIZE = 256;
  let _index = 0;
  AEVMEEPROM.prototype = Object.create(IEEPROMLoader.prototype);
  AEVMEEPROM.prototype.constructor = AEVMEEPROM;
  AEVMEEPROM.prototype.init = function(codec, regmodel) {
    console.log("aevm init");
    let deferred = Q.defer();
    this.aevmCodec = codec || regmodel.getModel()._codec;
    let self = this;
    this.getInfo().then(function(packet) {
      self.controllerInfo = packet;
      deferred.resolve();
    }).fail(function(er){
      deferred.reject(er);
    });
    return deferred.promise;
  };
  AEVMEEPROM.prototype.read = function() {
    let self = this;
    _index = 0;
    let deferred = Q.defer();
    let info = this.aevmCodec.getInterfaces().i2c.info;
    this.configure(info.config).then(function() {
      self
        ._aevmRead(
          {
            addr: info.regAddr || 0x00,
            deviceAddrs: info.deviceAddrs,
            unit: info.unit,
            flags: info.flags,
            _num_read_bytes: BLOCK_SIZE
            // read_register_protocol: info.read_register_protocol
          },
          ""
        )
        .then(function(readValue) {
          self.configure(info.reconfig);
          deferred.resolve(readValue);
        })
        .fail(function (err) {
          if (err.indexOf('-775')) {
            deferred.reject('No Device');
          } else {
            deferred.reject(err);
          }
        })
        .finally(function() {
          _index = 0;
        });
    })
    .fail(function(er){
      deferred.reject(er);
    });
    return deferred.promise;
  };

  AEVMEEPROM.prototype._aevmRead = function(regInfo, readData) {
    let deferred = Q.defer();
    that = this;
    regInfo.addr = "0x" + (_index * BLOCK_SIZE).toString(16);
    if ( _index * BLOCK_SIZE <= EEPROM_SIZE ){
    this.aevmCodec
      .readValue(regInfo)
      .then(function(readBuf) {
        let i = 0;
        let result = "";
        for (i = 0; i < readBuf.length; i++) {
          result += String.fromCharCode(readBuf[i]);
        }
        readData += result;
        if (readData.indexOf("#SOE") !== -1) {
          if (readData.indexOf("#EOE") !== -1) {
            deferred.resolve(readData);
          } else {
            _index++;
            that
              ._aevmRead(regInfo, readData)
              .then(function(readData) {
                deferred.resolve(readData);
              })
              .fail(function(er) {
                deferred.reject(er);
              });
          }
        } else {
          deferred.resolve("EEPROM is yet to be programmed");
        }
      })
      .fail(function(er) {
        deferred.reject(er);
      });
    } else {
      deferred.reject("EEPROM is programmed incorrectly");
    }

    return deferred.promise;
  };

  AEVMEEPROM.prototype.write = function(writeData) {
    _index = 0;
    let self = this;
    let deferred = Q.defer();
    let info = this.aevmCodec.getInterfaces().i2c.info;
    let dataArr = [];
    if (writeData.length > EEPROM_SIZE)
      deferred.reject(
        "Data exceeds the EEPROM size: " + EEPROM_SIZE + " bytes."
      );
    else {
      if (typeof writeData === "string") {
        for (i = 0; i < writeData.length; i++) {
          dataArr.push(writeData.charCodeAt(i));
        }
      } else {
        dataArr = writeData;
      }
      this.configure(info.config).then(function() {
        self
          ._aevmWrite(
            {
              addr: info.regAddr || 0x00,
              deviceAddrs: info.deviceAddrs,
              flags: info.flags,
              write_register_protocol: {}
            },
            dataArr
          )
          .then(function() {
            self.configure(info.reconfig);
            deferred.resolve("Successfully written to EEPROM");
          })
          .fail(function(err) {
            if (err.indexOf('-771')) {
              deferred.reject('Error: Write Protected jumper is not connected. '+ err);
            } else {
              deferred.reject(err);
            }
          })
          .finally(function() {
            _index = 0;
          });
      })
      .fail(function(er){
        deferred.reject(er);
      });
    }
    return deferred.promise;
  };

  AEVMEEPROM.prototype._aevmWrite = function(regInfo, writeData) {
    let self = this;
    let deferred = Q.defer();
    regInfo.addr = _index * BLOCK_SIZE;
    this.aevmCodec
      .writeValue(regInfo, writeData.slice(0, BLOCK_SIZE))
      .then(function(status) {
        writeData.splice(0, BLOCK_SIZE);
        if (writeData.length !== 0) {
          _index++;
          self
            ._aevmWrite(regInfo, writeData)
            .then(function() {
              deferred.resolve();
            })
            .fail(function(er) {
              deferred.reject(er);
            });
        } else {
          deferred.resolve();
        }
      })
      .fail(function(er) {
        deferred.reject(er);
      });

    return deferred.promise;
  };

  AEVMEEPROM.prototype.clearEEPROM = function() {
    let clearStr = Array(EEPROM_SIZE).fill(255);
    return this.write(clearStr);
  };

  AEVMEEPROM.prototype.configure = function(config_seq) {
    return this.aevmCodec.getInterfaces().i2c.ensureConfigured(config_seq);
  };

  AEVMEEPROM.prototype.getInfo = function() {
    return this.aevmCodec.getInterfaces().system.getInfo();
  };

  window.AEVMEEPROM = AEVMEEPROM;
})();
