function IEEPROMLoader() {}

IEEPROMLoader.prototype.readValue = function() {
  let deferred = Q.defer();
  let self = this;
  this.read()
    .then(function(result) {
      if (result !== "EEPROM is yet to be programmed") {
        let properties;
        let resObj = {};
        properties = result.split(";");
        properties.forEach(function(property) {
          if (property !== "#SOE" && property.indexOf("#EOE") === -1) {
            let x = property.split(":");
            resObj[x[0]] = x[1];
          }
        });
        result = resObj;
      }
      deferred.resolve(result);
    })
    .fail(function(er) {
      if (er === 'No Device') {
        deferred.resolve('No Device');
      } else {
        deferred.reject(er);
      }
    });
  return deferred.promise;
};

IEEPROMLoader.prototype.writeValue = function(eepromObjData) {
  let self = this;
  let writeString = "#SOE;";
  let deferred = Q.defer();
  self
    .readValue()
    .then(function(readData) {
      if (readData === "EEPROM is yet to be programmed") {
        eepromObjData.serial_number = self.updateLastModified("serial_number");
      } else {
        eepromObjData.serial_number = readData.serial_number;
      }
      return self.clearEEPROM();
    })
    .then(function() {
      for (let property in eepromObjData) {
        if (property === "last_modified") {
          eepromObjData[property] = self.updateLastModified(property);
        } else if (property === "fw_ver") {
          let fwDetails =  self.controllerInfo.FirmwareVersion;
          let firmwareVersion = fwDetails.major + '.' + fwDetails.minor + '.' + fwDetails.reversion + '.' + fwDetails.build;
          eepromObjData[property] = firmwareVersion;
        }
        writeString += property + ":" + eepromObjData[property] + ";";
      }
      writeString += "#EOE";
      return self.write(writeString);
    })
    .then(function() {
      return self.readValue();
    })
    .then(function(readData) {
      if (JSON.stringify(readData) === JSON.stringify(eepromObjData)) {
        // self.updateJson(self.deviceName, readData);
        deferred.resolve(eepromObjData);
      } else {
        deferred.reject("EEPROM write verification fails");
      }
    })
    .catch(function(er) {
      deferred.reject(er);
    });
  return deferred.promise;
};

IEEPROMLoader.prototype.getDateInfo = function() {
  function pad(data) {
    return data < 10 ? "0" + data : data;
  }
  function replaceIO(char) {
    return char === "I" ? "J" : char === "O" ? "P" : char;
  }
  let date = new Date(),
    year = date
      .getFullYear()
      .toString()
      .slice(2),
    month = replaceIO(String.fromCharCode(65 + date.getMonth())),
    dateInfo = pad(date.getDate()),
    hour = replaceIO(String.fromCharCode(65 + date.getHours())),
    minute = pad(date.getMinutes()),
    sec = pad(date.getSeconds());

  return year + month + dateInfo + hour + minute + sec;
};

IEEPROMLoader.prototype.updateLastModified = function(type) {
  let value;
  if (type === "serial_number") {
    value = "T" + this.getDateInfo();
  } else {
    value = this.getDateInfo();
  }
  return value;
};
IEEPROMLoader.prototype.init = function(codec, model) {};
IEEPROMLoader.prototype.read = function() {};
IEEPROMLoader.prototype.write = function(data) {};
IEEPROMLoader.prototype.configure = function(config_seq) {};
IEEPROMLoader.prototype.getInfo = function(config_seq) {
  this.controllerInfo = {
    FirmwareVersion: {}
  };
};
